def test_import_module():
    import pzp

    assert pzp


def test_import_ansi():
    from pzp import ansi

    assert ansi


def test_import_finder():
    from pzp import finder

    assert finder


def test_import_input():
    from pzp import input

    assert input


def test_import_keys():
    from pzp import keys

    assert keys


def test_import_screen():
    from pzp import screen

    assert screen
